@@ -1,26 +1,35 @@
 /* nettle-hash.c
- *
- * General hashing tool. */
 
-/* nettle, low-level cryptographics library
- *
- * Copyright (C) 2011 Niels Möller
- *  
- * The nettle library is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- * 
- * The nettle library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with the nettle library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02111-1301, USA.
- */
+   General hashing tool.
+
+   Copyright (C) 2011, 2013 Niels Möller
+
+   This file is part of GNU Nettle.
+
+   GNU Nettle is free software: you can redistribute it and/or
+   modify it under the terms of either:
+
+     * the GNU Lesser General Public License as published by the Free
+       Software Foundation; either version 3 of the License, or (at your
+       option) any later version.
+
+   or
+
+     * the GNU General Public License as published by the Free
+       Software Foundation; either version 2 of the License, or (at your
+       option) any later version.
+
+   or both in parallel, as here.
+
+   GNU Nettle is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received copies of the GNU General Public License and
+   the GNU Lesser General Public License along with this program.  If
+   not, see http://www.gnu.org/licenses/.
+*/
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -108,7 +117,7 @@ digest_file(const struct nettle_hash *alg,
   else
     {
       unsigned i;
-      char *hex = xalloc(BASE16_ENCODE_LENGTH(8) + 1);
+      char hex[BASE16_ENCODE_LENGTH(8) + 1];
       for (i = 0; i + 8 < digest_length; i += 8)
 	{
 	  base16_encode_update(hex, 8, digest + i);
@@ -118,7 +127,6 @@ digest_file(const struct nettle_hash *alg,
       base16_encode_update(hex, digest_length - i, digest + i);
       hex[BASE16_ENCODE_LENGTH(digest_length - i)] = 0;
       printf("%s %s\n", hex, alg->name);
-      free(hex);
     }
   
   free(digest);
